#include "Canvas.h"
#include "NullDrawBehavior.h"

Canvas::Canvas(QWidget *parent)
    :
    QGLWidget(parent),
    behavior(new NullDrawBehavior())
{
}

void Canvas::setDrawBehavior(DrawBehavior *behavior)
{
    this->behavior = behavior;
}

DrawBehavior* Canvas::getDrawBehavior()
{
    return behavior;
}

void Canvas::autoScale()
{
    behavior->autoScale();
}

void Canvas::setScale(float scale)
{
    behavior->setScale(scale);
}

float Canvas::getScale()
{
    return behavior->getScale();
}

void Canvas::initializeGL()
{
    qglClearColor(Qt::black);
}

void Canvas::paintGL()
{
    behavior->draw();
}

void Canvas::resizeGL(int width, int height)
{
    float x = width / 2;
    float y = height / 2;
    
    if (width >= height)
        glViewport(x - height / 2, y - height / 2, height, height);
    else
        glViewport(x - width / 2, y - width / 2, width, width);        
}
