#include "MyModel.h"
#include <algorithm>

MyModel::MyModel()
    :
    db(nullptr)
{
}

void MyModel::notify()
{
    for (auto &v : views)
        v->update();
}

void MyModel::addView(View *view)
{
    views.push_back(view);
}

void MyModel::removeView(View *view)
{
    views.erase(std::remove(views.begin(), views.end(), view),
                views.end());
}

void MyModel::loadDatabase(const std::string &path)
{
    db = rt_dirbuild(path.c_str(), nullptr, 0);
    notify();
}

bool MyModel::isLoaded() const
{
    return db != RTI_NULL;
}

std::string MyModel::getTitle() const
{
    if (!isLoaded())
        return "";

    return std::string(db->rti_dbip->dbi_title);
}

std::vector<std::string> MyModel::getElements() const
{
    std::vector<std::string> ret;

    if (!isLoaded())
        return ret;

    rt_init_resource(&rt_uniresource, 0, db);
    db_update_nref(db->rti_dbip, &rt_uniresource);

    for (int i = 0; i < RT_DBNHASH; i++) {

        for (directory *dir = db->rti_dbip->dbi_Head[i];
             dir != RT_DIR_NULL;
             dir = dir->d_forw) {

            if (dir->d_nref == 0)
                ret.push_back(std::string(dir->d_namep));
        }
    }

    return ret;
}

std::vector<std::string> MyModel::getExtends(const std::string &elem) const
{
    std::vector<std::string> ret;

    if (!isLoaded() || elem.size() < 0)
        return ret;

    directory *dir = db_lookup(db->rti_dbip, elem.c_str(), true);

    if (dir == RT_DIR_NULL)
        return ret;

    rt_db_internal intern;
    resource *m_resp = nullptr;
    
    int id = rt_db_get_internal(&intern, dir, db->rti_dbip, 0, m_resp);

    switch(id) {
    case ID_TOR: // 1
        ret.push_back("name: Torus");
        break;

    case ID_TGC: // 2
        ret.push_back("name: Cone");
        break;

    case ID_ELL: // 3
        ret.push_back("name: Ellipsoid");
        break;

    case ID_ARB8: // 4
        ret.push_back("name: Arb8");
        break;

    case ID_HALF: // 6
        ret.push_back("name: Halfspace");
        break;

    case ID_SPH: // 10
        ret.push_back("name: Sphere");
        break;

    case ID_NMG: // 11
        ret.push_back("name: NonManifoldGeometry");
        break;

    case ID_PARTICLE: // 16
        ret.push_back("name: Particle");
        break;

    case ID_RPC: // 17
        ret.push_back("name: ParabolicCylinder");
        break;

    case ID_RHC: // 18
        ret.push_back("name: HyperbolicCylinder");
        break;

    case ID_EPA: // 19
        ret.push_back("name: Paraboloid");
        break;

    case ID_EHY: // 20
        ret.push_back("name: Hyperboloid");
        break;

    case ID_ETO: // 21
        ret.push_back("name: EllipticalTorus");
        break;

    case ID_BOT: // 30
        ret.push_back("name: BagOfTriangles");
        break;

    case ID_COMBINATION: // 31
        ret.push_back("name: Combination");
        break;

    default:
        ret.push_back("name: Unknown");
    }

    return ret;
}
