#include "MyWindow.h"

#include "ArbDraw.h"
#include <raytrace.h>

#include <QApplication>
#include <QDesktopWidget>
#include <QFileDialog>
#include <QMessageBox>

MyWindow::MyWindow(Model* model, Controller *controller,
                   QWidget *parent)
    :
    View(parent),
    model(model),
    controller(controller),
    elemWindow(this),
    extendsWindow(&elemWindow),
    list(&elemWindow),
    extendsList(&extendsWindow),
    extendsButton(&elemWindow),
    canvas(this)
{
    // call setups.
    createAction();
    createMenu();
    update();

    // MyWidow setup.
    setWindowTitle("brlcad");

    resize(QApplication::desktop()->size());
    QRect screen = QApplication::desktop()->screenGeometry();
    int x = (screen.width() - width()) / 2;
    int y = (screen.height() - height()) / 2;
    move(x, y);

    // elemWindow setup.
    elemWindow.setFixedSize(400, 400);
    elemWindow.setWindowTitle("Top elements");

    // extendsWindow setup.
    extendsWindow.setFixedSize(400, 400);

    // Top element list setup.
    list.resize(350, 250);
    list.move(elemWindow.rect().center() - list.rect().center());

    // Extends list setup.
    extendsList.resize(350, 250);
    extendsList.move(extendsWindow.rect().center() -
                     extendsList.rect().center());    

    // Extends button setup.
    extendsButton.setText("get extends");
    extendsButton.move(elemWindow.rect().center().x() -
                       extendsButton.rect().center().x(),
                       elemWindow.height() - 50);
    
    connect(&extendsButton, SIGNAL(released()),
            this, SLOT(getExtendsAction()));

    // Canvas example
    rt_arb_internal *arb = new rt_arb_internal();

    arb->pt[0][0] = -0.5f;
    arb->pt[0][1] = -0.5f;
    arb->pt[0][2] = 0.0f;
    
    arb->pt[1][0] = 0.5f;
    arb->pt[1][1] = -0.5f;
    arb->pt[1][2] = 0.0f;
    
    arb->pt[2][0] = 0.5f;
    arb->pt[2][1] = 0.5f;
    arb->pt[2][2] = 0.0f;
    
    arb->pt[3][0] = -0.5f;
    arb->pt[3][1] = 0.5f;
    arb->pt[3][2] = 0.5f;

    arb->pt[4][0] = -0.7f;
    arb->pt[4][1] = -0.7f;
    arb->pt[4][2] = 0.7f;
    
    arb->pt[5][0] = 0.7f;
    arb->pt[5][1] = -0.7f;
    arb->pt[5][2] = 0.7f;
    
    arb->pt[6][0] = 0.7f;
    arb->pt[6][1] = 0.7f;
    arb->pt[6][2] = 0.7f;
    
    arb->pt[7][0] = -0.7f;
    arb->pt[7][1] = 0.7f;
    arb->pt[7][2] = 0.7f;

    ArbDraw *behavior = new ArbDraw(arb);
    canvas.setDrawBehavior(behavior);
}

void MyWindow::createMenu()
{
    brlcadMenu = menuBar()->addMenu("&BRL-CAD");
    brlcadMenu->addAction(brlcadTitleAction);
    brlcadMenu->addAction(brlcadShowElements);    
}

void MyWindow::createAction()
{
    brlcadTitleAction = new QAction("&Open", this);
    connect(brlcadTitleAction, &QAction::triggered,
            this, &MyWindow::openDatabaseAction);

    brlcadShowElements = new QAction("&show elements", this);
    connect(brlcadShowElements, &QAction::triggered,
            this, &MyWindow::showElementsAction);
}

void MyWindow::resizeEvent(QResizeEvent *event)
{
    canvas.resize(event->size());
}

void MyWindow::update()
{
    if (model->isLoaded()) {
        brlcadTitleAction->setEnabled(false);
        brlcadShowElements->setEnabled(true);
    }
    else {
        brlcadTitleAction->setEnabled(true);
        brlcadShowElements->setEnabled(false);        
    }
}

void MyWindow::errorWhileLoading()
{
    QMessageBox::information(this, "", "error while loading database");
}

void MyWindow::openDatabaseAction()
{
    const QString &input = QFileDialog::getOpenFileName(
                        this,
                        "Open database",
                        "",
                        "Database (*.g);;All files (*)");

    controller->loadDatabase(input.toStdString());
    setWindowTitle(QString::fromStdString(model->getTitle()));
}

void MyWindow::showElementsAction()
{
    elemWindow.show();
    std::vector<std::string> elems = model->getElements();

    list.clear();
    
    for (int i = 0; i < elems.size(); i++)
        list.insertItem(i, QString::fromStdString(elems[i]));
}

void MyWindow::getExtendsAction()
{
    QList<QListWidgetItem*> sl = list.selectedItems();
    std::string arg = sl[0]->text().toStdString();
    std::vector<std::string> e = model->getExtends(arg);

    extendsWindow.show();
    extendsWindow.setWindowTitle(QString::fromStdString(arg));
    extendsList.clear();

    for (int i = 0; i < e.size(); i++)
        extendsList.insertItem(i, QString::fromStdString(e[i]));
}
